package de.unikoblenz.fgbks.base.utils.boundary.impl;

import de.unikoblenz.fgbks.base.utils.boundary.AbstractBoundary;
import de.unikoblenz.fgbks.base.utils.boundary.Boundary;
import de.unikoblenz.fgbks.base.utils.boundary.BoundaryParseException;
import de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheck;
import de.unikoblenz.fgbks.base.utils.boundary.checker.BoundaryCheckType;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Objects;
import org.apache.commons.lang3.Validate;

public class StringBoundary extends AbstractBoundary<String> {

  private boolean matchNoneOfValues;
  private boolean matchAny;
  private String[] values;
  private int[] valuesHashes;

  public StringBoundary(String text) {
    super(text, String.class);
  }

  @Override
  public int compareTo(Boundary o) {
    return o.getText().compareTo(o.getText());
  }

  @Override
  protected void parse(String text) {
    matchNoneOfValues = false;
    matchAny = false;
    try {
      matchAny = Validate.notNull(text).isEmpty();
      if (matchAny) {
        values = new String[0];
      } else {
        matchNoneOfValues = text.toLowerCase().startsWith("not");
        text = text.replace('(', '\0').replace(')', '\0').trim();
        if (matchNoneOfValues) {
          text = text.replaceFirst("not", "").trim();
        }
        values = text.split(",");
        valuesHashes = new int[values.length];
        for (int i = 0; i < values.length; i++) {
          values[i] = values[i].replace('"', '\0').trim();
          valuesHashes[i] = Objects.hashCode(values[i]);
        }
      }
    } catch (Exception e) {
      throw new BoundaryParseException("String value is not valid: " + this.getText());
    }
  }

  @Override
  protected void validate() {
    Validate.noNullElements(values);
  }

  @Override
  protected HashMap<BoundaryCheckType, BoundaryCheck> getChecker() {
    return new HashMap<>();
  }

  @Override
  protected String getParsedText() {
    return (matchNoneOfValues ? "not" : "") + Arrays.toString(values);
  }
}
